from model_utils import Choices

from django.utils.translation import gettext_lazy as _

from .country import COUNTRIES


__all__ = [
    "OTHER_CURRENCIES",
    "MARKETPLACE_CURRENCIES",
    "CURRENCIES",
    "USER_CURRENCIES",
    "INTERNAL_CURRENCIES",
    "CURRENCY_CODE_TO_NAME",
    "CURRENCY_NAME_TO_CODE",
    "CURRENCIES_BY_COUNTRY_CODE",
]


OTHER_CURRENCIES = Choices(
    ("AED", _("Дирхам (Оаэ)")),
    ("AFN", _("Афгани")),
    ("ALL", _("Лек")),
    ("AMD", _("Армянский Драм")),
    ("ANG", _("Нидерландский Антильский Гульден")),
    ("AOA", _("Кванза")),
    ("ARS", _("Аргентинское Песо")),
    ("AWG", _("Арубанский Флорин")),
    ("AZN", _("Азербайджанский Манат")),
    ("BAM", _("Конвертируемая Марка")),
    ("BBD", _("Барбадосский Доллар")),
    ("BDT", _("Така")),
    ("BGN", _("Болгарский Лев")),
    ("BHD", _("Бахрейнский Динар")),
    ("BIF", _("Бурундийский Франк")),
    ("BMD", _("Бермудский Доллар")),
    ("BND", _("Брунейский Доллар")),
    ("BOB", _("Боливиано")),
    ("BRL", _("Бразильский Реал")),
    ("BSD", _("Багамский Доллар")),
    ("BTN", _("Нгултрум")),
    ("BWP", _("Пула")),
    ("BYR", _("Белорусский Рубль")),
    ("BZD", _("Белизский Доллар")),
    ("CDF", _("Конголезский Франк")),
    ("CLP", _("Чилийское Песо")),
    ("COP", _("Колумбийское Песо")),
    ("CRC", _("Костариканский Колон")),
    ("CUP", _("Кубинское Песо")),
    ("CVE", _("Эскудо Кабо-Верде")),
    ("DJF", _("Франк Джибути")),
    ("DOP", _("Доминиканское Песо")),
    ("DZD", _("Алжирский Динар")),
    ("EGP", _("Египетский Фунт")),
    ("ERN", _("Накфа")),
    ("ETB", _("Эфиопский Быр")),
    ("FJD", _("Доллар Фиджи")),
    ("FKP", _("Фунт Фолклендских Островов")),
    ("GEL", _("Ларистанский")),
    ("GHS", _("Седи")),
    ("GIP", _("Гибралтарский Фунт")),
    ("GMD", _("Даласи")),
    ("GNF", _("Гвинейский Франк")),
    ("GTQ", _("Кетсаль")),
    ("GYD", _("Гайанский Доллар")),
    ("HNL", _("Лемпира")),
    ("HRK", _("Куна")),
    ("HTG", _("Гурд")),
    ("IQD", _("Иракский Динар")),
    ("IRR", _("Иранский Риал")),
    ("ISK", _("Исландская Крона")),
    ("JMD", _("Ямайский Доллар")),
    ("JOD", _("Иорданский Динар")),
    ("KES", _("Кенийский Шиллинг")),
    ("KGS", _("Сом")),
    ("KHR", _("Риель")),
    ("KMF", _("Франк Комор")),
    ("KPW", _("Северокорейская Вона")),
    ("KRW", _("Вона")),
    ("KWD", _("Кувейтский Динар")),
    ("KYD", _("Доллар Островов Кайман")),
    ("KZT", _("Тенге")),
    ("LAK", _("Кип")),
    ("LBP", _("Ливанский Фунт")),
    ("LKR", _("Шри-Ланкийская Рупия")),
    ("LRD", _("Либерийский Доллар")),
    ("LSL", _("Лоти")),
    ("LTL", _("Литовский Лит")),
    ("LYD", _("Ливийский Динар")),
    ("MAD", _("Марокканский Дирхам")),
    ("MDL", _("Молдавский Лей")),
    ("MGA", _("Малагасийский Ариари")),
    ("MKD", _("Македонский Денар")),
    ("MMK", _("Кьят")),
    ("MNT", _("Тугрик")),
    ("MOP", _("Патака")),
    ("MRO", _("Угия")),
    ("MUR", _("Маврикийская Рупия")),
    ("MVR", _("Руфия")),
    ("MWK", _("Малавийская Квача")),
    ("MXN", _("Мексиканское Песо")),
    ("MZN", _("Мозамбикский Метикал")),
    ("NAD", _("Доллар Намибии")),
    ("NGN", _("Найра")),
    ("NIO", _("Золотая Кордоба")),
    ("NPR", _("Непальская Рупия")),
    ("OMR", _("Оманский Риал")),
    ("PAB", _("Бальбоа")),
    ("PGK", _("Кина")),
    ("PKR", _("Пакистанская Рупия")),
    ("PYG", _("Гуарани")),
    ("QAR", _("Катарский Риал")),
    ("RON", _("Румынский Лей")),
    ("RSD", _("Сербский Динар")),
    ("RUR", _("Российский Рубль")),
    ("RWF", _("Франк Руанды")),
    ("SAR", _("Саудовский Риял")),
    ("SBD", _("Доллар Соломоновых Островов")),
    ("SCR", _("Сейшельская Рупия")),
    ("SDG", _("Суданский Фунт")),
    ("SHP", _("Фунт Святой Елены")),
    ("SLL", _("Леоне")),
    ("SOS", _("Сомалийский Шиллинг")),
    ("SRD", _("Суринамский Доллар")),
    ("STD", _("Добра")),
    ("SYP", _("Сирийский Фунт")),
    ("SZL", _("Лилангени")),
    ("TJS", _("Сомони")),
    ("TMT", _("Туркменский Новый Манат")),
    ("TND", _("Тунисский Динар")),
    ("TOP", _("Паанга")),
    ("TRY", _("Турецкая Лира")),
    ("TTD", _("Доллар Тринидада И Тобаго")),
    ("TZS", _("Танзанийский Шиллинг")),
    ("UAH", _("Гривна")),
    ("UGX", _("Угандийский Шиллинг")),
    ("UYU", _("Уругвайское Песо")),
    ("UZS", _("Узбекский Сум")),
    ("VEF", _("Боливар")),
    ("VUV", _("Вату")),
    ("WST", _("Тала")),
    ("XAF", _("Франк Кфа Веас")),
    ("XCD", _("Восточно-Карибский Доллар")),
    ("XOF", _("Франк Кфа Всеао")),
    ("XPF", _("Франк Кфп")),
    ("YER", _("Йеменский Риал")),
    ("ZMW", _("Замбийская Квача")),
    ("ZWL", _("Доллар Зимбабве")),
)

MARKETPLACE_CURRENCIES = Choices(
    ("USD", _("Доллар США")),
    ("RUB", _("Российский Рубль")),
    ("EUR", _("Евро")),
    ("GBP", _("Фунт Стерлингов")),
    ("AUD", _("Австралийский Доллар")),
    ("CNY", _("Китайский Юань")),
    ("PLN", _("Злотый")),
    ("CHF", _("Швейцарский Франк")),
    ("JPY", _("Иена")),
    ("IDR", _("Рупия")),
    ("VND", _("Донг")),
    ("NOK", _("Норвежская Крона")),
    ("SGD", _("Сингапурский Доллар")),
    ("SEK", _("Шведская Крона")),
    ("MYR", _("Малайзийский Ринггит")),
    ("THB", _("Бат")),
    ("NZD", _("Новозеландский Доллар")),
    ("HUF", _("Форинт")),
    ("HKD", _("Гонконгский Доллар")),
    ("INR", _("Индийская Рупия")),
    ("PHP", _("Филиппинское Песо")),
    ("DKK", _("Датская Крона")),
    ("TWD", _("Новый Тайваньский Доллар")),
    ("PEN", _("Соль")),
    ("ZAR", _("Рэнд")),
    ("CAD", _("Канадский Доллар")),
    ("CZK", _("Чешская Крона")),
    ("ILS", _("Новый Израильский Шекель")),
)

CURRENCIES = MARKETPLACE_CURRENCIES + OTHER_CURRENCIES

CURRENCY_CODE_TO_NAME = dict(CURRENCIES)

CURRENCY_NAME_TO_CODE = dict((v, k) for k, v in CURRENCY_CODE_TO_NAME.items())

USER_CURRENCIES = CURRENCIES.subset("RUB", "USD")

INTERNAL_CURRENCIES = CURRENCIES.subset("USD", "EUR")

CURRENCIES_BY_COUNTRY_CODE = {
    COUNTRIES.BD: ("BDT",),
    COUNTRIES.BE: ("EUR",),
    COUNTRIES.BF: ("XOF",),
    COUNTRIES.BG: ("BGN",),
    COUNTRIES.BA: ("BAM",),
    COUNTRIES.BB: ("BBD",),
    COUNTRIES.WF: ("XPF",),
    COUNTRIES.BL: ("EUR",),
    COUNTRIES.BM: ("BMD",),
    COUNTRIES.BN: ("BND",),
    COUNTRIES.BO: ("BOB",),
    COUNTRIES.BH: ("BHD",),
    COUNTRIES.BI: ("BIF",),
    COUNTRIES.BJ: ("XOF",),
    COUNTRIES.BT: ("BTN", "INR"),
    COUNTRIES.JM: ("JMD",),
    COUNTRIES.BV: ("NOK",),
    COUNTRIES.BW: ("BWP",),
    COUNTRIES.WS: ("WST",),
    COUNTRIES.BQ: ("USD",),
    COUNTRIES.BR: ("BRL",),
    COUNTRIES.BS: ("BSD",),
    COUNTRIES.JE: ("GBP",),
    COUNTRIES.BY: ("BYR",),
    COUNTRIES.BZ: ("BZD",),
    COUNTRIES.RU: ("RUB",),
    COUNTRIES.RW: ("RWF",),
    COUNTRIES.RS: ("RSD",),
    COUNTRIES.TL: ("USD",),
    COUNTRIES.RE: ("EUR",),
    COUNTRIES.TM: ("TMT",),
    COUNTRIES.TJ: ("TJS",),
    COUNTRIES.RO: ("RON",),
    COUNTRIES.TK: ("NZD",),
    COUNTRIES.GW: ("XOF",),
    COUNTRIES.GU: ("USD",),
    COUNTRIES.GT: ("GTQ",),
    COUNTRIES.GS: ("GBP",),
    COUNTRIES.GR: ("EUR",),
    COUNTRIES.GQ: ("XAF",),
    COUNTRIES.GP: ("EUR",),
    COUNTRIES.JP: ("JPY",),
    COUNTRIES.GY: ("GYD",),
    COUNTRIES.GG: ("GBP",),
    COUNTRIES.GF: ("EUR",),
    COUNTRIES.GE: ("GEL",),
    COUNTRIES.GD: ("XCD",),
    COUNTRIES.GB: ("GBP",),
    COUNTRIES.GA: ("XAF",),
    COUNTRIES.SV: ("USD",),
    COUNTRIES.GN: ("GNF",),
    COUNTRIES.GM: ("GMD",),
    COUNTRIES.GL: ("DKK",),
    COUNTRIES.GI: ("GIP",),
    COUNTRIES.GH: ("GHS",),
    COUNTRIES.OM: ("OMR",),
    COUNTRIES.TN: ("TND",),
    COUNTRIES.IL: ("ILS",),
    COUNTRIES.JO: ("JOD",),
    COUNTRIES.HR: ("HRK",),
    COUNTRIES.HT: ("HTG", "USD"),
    COUNTRIES.HU: ("HUF",),
    COUNTRIES.HK: ("HKD",),
    COUNTRIES.HN: ("HNL",),
    COUNTRIES.HM: ("AUD",),
    COUNTRIES.VE: ("VEF",),
    COUNTRIES.PR: ("USD",),
    COUNTRIES.PS: (),
    COUNTRIES.PW: ("USD",),
    COUNTRIES.PT: ("EUR",),
    COUNTRIES.SJ: ("NOK",),
    COUNTRIES.PY: ("PYG",),
    COUNTRIES.IQ: ("IQD",),
    COUNTRIES.PA: ("PAB", "USD"),
    COUNTRIES.PF: ("XPF",),
    COUNTRIES.PG: ("PGK",),
    COUNTRIES.PE: ("PEN",),
    COUNTRIES.PK: ("PKR",),
    COUNTRIES.PH: ("PHP",),
    COUNTRIES.PN: ("NZD",),
    COUNTRIES.PL: ("PLN",),
    COUNTRIES.PM: ("EUR",),
    COUNTRIES.ZM: ("ZMW",),
    COUNTRIES.EH: ("MAD",),
    COUNTRIES.EE: ("EUR",),
    COUNTRIES.EG: ("EGP",),
    COUNTRIES.ZA: ("ZAR",),
    COUNTRIES.EC: ("USD",),
    COUNTRIES.IT: ("EUR",),
    COUNTRIES.VN: ("VND",),
    COUNTRIES.SB: ("SBD",),
    COUNTRIES.ET: ("ETB",),
    COUNTRIES.SO: ("SOS",),
    COUNTRIES.ZW: ("USD", "ZAR", "BWP", "GBP", "EUR"),
    COUNTRIES.SA: ("SAR",),
    COUNTRIES.ES: ("EUR",),
    COUNTRIES.ER: ("ETB", "ERN"),
    COUNTRIES.ME: ("EUR",),
    COUNTRIES.MD: ("MDL",),
    COUNTRIES.MG: ("MGA",),
    COUNTRIES.MF: ("EUR",),
    COUNTRIES.MA: ("MAD",),
    COUNTRIES.MC: ("EUR",),
    COUNTRIES.UZ: ("UZS",),
    COUNTRIES.MM: ("MMK",),
    COUNTRIES.ML: ("XOF",),
    COUNTRIES.MO: ("MOP",),
    COUNTRIES.MN: ("MNT",),
    COUNTRIES.MH: ("USD",),
    COUNTRIES.MK: ("MKD",),
    COUNTRIES.MU: ("MUR",),
    COUNTRIES.MT: ("EUR",),
    COUNTRIES.MW: ("MWK",),
    COUNTRIES.MV: ("MVR",),
    COUNTRIES.MQ: ("EUR",),
    COUNTRIES.MP: ("USD",),
    COUNTRIES.MS: ("XCD",),
    COUNTRIES.MR: ("MRO",),
    COUNTRIES.IM: ("GBP",),
    COUNTRIES.UG: ("UGX",),
    COUNTRIES.MY: ("MYR",),
    COUNTRIES.MX: ("MXN",),
    COUNTRIES.AT: ("EUR",),
    COUNTRIES.FR: ("EUR",),
    COUNTRIES.IO: ("USD",),
    COUNTRIES.SH: ("SHP",),
    COUNTRIES.FI: ("EUR",),
    COUNTRIES.FJ: ("FJD",),
    COUNTRIES.FK: ("FKP",),
    COUNTRIES.FM: ("USD",),
    COUNTRIES.FO: ("DKK",),
    COUNTRIES.NI: ("NIO",),
    COUNTRIES.NL: ("EUR",),
    COUNTRIES.NO: ("NOK",),
    COUNTRIES.NA: ("NAD", "ZAR"),
    COUNTRIES.NC: ("XPF",),
    COUNTRIES.NE: ("XOF",),
    COUNTRIES.NF: ("AUD",),
    COUNTRIES.NG: ("NGN",),
    COUNTRIES.NZ: ("NZD",),
    COUNTRIES.NP: ("NPR",),
    COUNTRIES.NR: ("AUD",),
    COUNTRIES.NU: ("NZD",),
    COUNTRIES.CK: ("NZD",),
    COUNTRIES.CI: ("XOF",),
    COUNTRIES.CH: ("CHF",),
    COUNTRIES.CO: ("COP",),
    COUNTRIES.CN: ("CNY",),
    COUNTRIES.CM: ("XAF",),
    COUNTRIES.CL: ("CLP",),
    COUNTRIES.CC: ("AUD",),
    COUNTRIES.CA: ("CAD",),
    COUNTRIES.LB: ("LBP",),
    COUNTRIES.CG: ("XAF",),
    COUNTRIES.CF: ("XAF",),
    COUNTRIES.CD: ("CDF",),
    COUNTRIES.CZ: ("CZK",),
    COUNTRIES.CY: ("EUR",),
    COUNTRIES.CX: ("AUD",),
    COUNTRIES.CR: ("CRC",),
    COUNTRIES.CW: ("ANG",),
    COUNTRIES.CV: ("CVE",),
    COUNTRIES.CU: ("CUP", "CUC"),
    COUNTRIES.SZ: ("SZL",),
    COUNTRIES.SY: ("SYP",),
    COUNTRIES.SX: ("ANG",),
    COUNTRIES.KG: ("KGS",),
    COUNTRIES.KE: ("KES",),
    COUNTRIES.SR: ("SRD",),
    COUNTRIES.KI: ("AUD",),
    COUNTRIES.KH: ("KHR",),
    COUNTRIES.KN: ("XCD",),
    COUNTRIES.KM: ("KMF",),
    COUNTRIES.ST: ("STD",),
    COUNTRIES.SK: ("EUR",),
    COUNTRIES.KR: ("KRW",),
    COUNTRIES.SI: ("EUR",),
    COUNTRIES.KP: ("KPW",),
    COUNTRIES.KW: ("KWD",),
    COUNTRIES.SN: ("XOF",),
    COUNTRIES.SM: ("EUR",),
    COUNTRIES.SL: ("SLL",),
    COUNTRIES.SC: ("SCR",),
    COUNTRIES.KZ: ("KZT",),
    COUNTRIES.KY: ("KYD",),
    COUNTRIES.SG: ("SGD",),
    COUNTRIES.SE: ("SEK",),
    COUNTRIES.SD: ("SDG",),
    COUNTRIES.DO: ("DOP",),
    COUNTRIES.DM: ("XCD",),
    COUNTRIES.DJ: ("DJF",),
    COUNTRIES.DK: ("DKK",),
    COUNTRIES.VG: ("USD",),
    COUNTRIES.DE: ("EUR",),
    COUNTRIES.YE: ("YER",),
    COUNTRIES.DZ: ("DZD",),
    COUNTRIES.US: ("USD",),
    COUNTRIES.UY: ("UYU",),
    COUNTRIES.YT: ("EUR",),
    COUNTRIES.UM: ("USD",),
    COUNTRIES.TZ: ("TZS",),
    COUNTRIES.LC: ("XCD",),
    COUNTRIES.LA: ("LAK",),
    COUNTRIES.TV: ("TVD", "AUD"),
    COUNTRIES.TW: ("TWD",),
    COUNTRIES.TT: ("TTD",),
    COUNTRIES.TR: ("TRY",),
    COUNTRIES.LK: ("LKR",),
    COUNTRIES.LI: ("CHF",),
    COUNTRIES.LV: ("EUR",),
    COUNTRIES.TO: ("TOP",),
    COUNTRIES.LT: ("LTL",),
    COUNTRIES.LU: ("EUR",),
    COUNTRIES.LR: ("LRD",),
    COUNTRIES.LS: ("LSL", "ZAR"),
    COUNTRIES.TH: ("THB",),
    COUNTRIES.TF: ("EUR",),
    COUNTRIES.TG: ("XOF",),
    COUNTRIES.TD: ("XAF",),
    COUNTRIES.TC: ("USD",),
    COUNTRIES.LY: ("LYD",),
    COUNTRIES.VA: ("EUR",),
    COUNTRIES.VC: ("XCD",),
    COUNTRIES.AE: ("AED",),
    COUNTRIES.AD: ("EUR",),
    COUNTRIES.AG: ("XCD",),
    COUNTRIES.AF: ("AFN",),
    COUNTRIES.AI: ("XCD",),
    COUNTRIES.VI: ("USD",),
    COUNTRIES.IS: ("ISK",),
    COUNTRIES.IR: ("IRR",),
    COUNTRIES.AM: ("AMD",),
    COUNTRIES.AL: ("ALL",),
    COUNTRIES.AO: ("AOA",),
    COUNTRIES.AN: ("ANG",),
    COUNTRIES.AQ: (),
    COUNTRIES.AS: ("USD",),
    COUNTRIES.AR: ("ARS",),
    COUNTRIES.AU: ("AUD",),
    COUNTRIES.VU: ("VUV",),
    COUNTRIES.AW: ("AWG",),
    COUNTRIES.IN: ("INR",),
    COUNTRIES.AX: ("EUR",),
    COUNTRIES.AZ: ("AZN",),
    COUNTRIES.IE: ("EUR",),
    COUNTRIES.ID: ("IDR",),
    COUNTRIES.UA: ("UAH",),
    COUNTRIES.QA: ("QAR",),
    COUNTRIES.MZ: ("MZN",),
}



from zs_utils.api.ebay.base_api import EbayAPI


class GetAppRateLimits(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/developer/analytics/resources/rate_limit/methods/getRateLimits
    """

    http_method = "GET"
    resource_method = "developer/analytics/v1_beta/rate_limit"
    allowed_params = ["api_name", "api_context"]


class GetUserRateLimits(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/developer/analytics/resources/user_rate_limit/methods/getUserRateLimits
    """

    http_method = "GET"
    resource_method = "developer/analytics/v1_beta/user_rate_limit"
    allowed_params = ["api_name", "api_context"]



import uuid
import datetime

from zs_utils.exceptions import CustomException


class DataCompareException(CustomException):
    pass


def get_dict_field_value(field: str, data: dict):
    """
    Получение значения поля 'field' словаря 'data'.
    Синтаксис аргумента 'field':
        -- символ '.' позволяет получить элемент из вложенного словаря или списка;
        -- символ ':' позволяет получить определенный элемент списка.
    Примеры работы:
        Пусть data = {'a': 1, 'b': {'d': 2}, 'c': [{'e': 3}, {'e': 4, 'f': 5}]}.
        Результат при 'field'='a': 1.
        Результат при 'field'='b': {'d': 2}.
        Результат при 'field'='b.d': 2.
        Результат при 'field'='c': [{'e': 3}, {'e': 4, 'f': 5}].
        Результат при 'field'='c.e': [3, 4].
        Результат при 'field'='c.f': [None, 5].
        Результат при 'field'='c:0': {'e': 3}.
    """

    exists = True

    for i, key in enumerate(field.split(".")):
        if ":" in key:
            key, num = key.split(":")
        else:
            num = None

        try:
            if num:
                data = data[key][int(num)]
            else:
                data = data[key]
        except (KeyError, TypeError, IndexError):
            exists = False
            data = None
            break

        if isinstance(data, list):
            field = ".".join(field.split(".")[i + 1 :])

            if field:
                if data:
                    data = [get_dict_field_value(field, item)[1] if isinstance(item, dict) else None for item in data]
                else:
                    data = None
                    exists = False

            break

    return exists, data


def dict_field_exists(*, field: str, data: dict):
    """
    Проверка, можно ли найти поле 'field' в словаре 'data'
    """

    return get_dict_field_value(field, data)[0]


def dict_fields_exist(*, fields: list, data: dict, raise_exception: bool = False):
    """
    Проверка наличия полей 'fields' в словаре 'data'
    """

    errors = {}

    for field in fields:
        if not dict_field_exists(field=field, data=data):
            errors[field] = f"Поле '{field}' не найдено."

    if errors and raise_exception:
        raise DataCompareException(message_dict=errors)

    return errors


def compare_values(a, b) -> bool:
    """
    Сравнение двух объектов на равенство.
    Допустимые типы аргументов: dict, list, str, int, float, bool, NoneType, datetime.
    """

    for value in [a, b]:
        assert type(value) in [
            dict,
            list,
            str,
            int,
            float,
            bool,
            type(None),
            datetime.datetime,
            uuid.UUID,
        ], f"Недопустимый тип {type(value)} значения '{value}'."

    values = [a, b]
    types = [type(a), type(b)]

    if dict in types:
        if isinstance(b, dict):
            # Делаем, так чтобы 'a' являлся словарем
            a, b = b, a

        if not isinstance(b, dict):
            # None и {} считаются равными
            return (b is None) and (not a)
        else:
            # Случай, когда оба значения -- словари
            if (not a) and (not b):
                # Два пустых словаря
                return True
            if set(list(a.keys())) == set(list(b.keys())):
                return all(compare_values(a[key], b[key]) for key in a.keys())
    elif types == [list, list]:
        # Списки сравниваются поэлементно
        if len(a) != len(b):
            # Равенство списков разной длины не считается возможным
            return False

        if all([(type(x) in [str, int, float]) for x in a + b]):
            try:
                # Сортировка таким образом, чтобы массив чисел сортировался так же,
                # как тот же массив с элементами, приведенными к типу str
                a = sorted(a, key=str)
                b = sorted(b, key=str)
            except TypeError:
                pass

        # Рекурсивное сравнение пар элементов списков
        return all(compare_values(a_i, b_i) for a_i, b_i in zip(a, b))
    elif (bool in types) or (None in values):
        if (int in types) or (float in types) or (str in types):
            # Равенство между bool или None с числами и строками не считается возможным
            return False
        elif (list in types) and (bool in types):
            # Равенство между bool со списком не считается возможным
            return False

        # Приведение обоих значений к bool
        return bool(a) == bool(b)

    try:
        # Попытка привести оба значения к типу float
        return float(a) == float(b)
    except (TypeError, ValueError):
        pass

    if set(types) == {uuid.UUID, str}:
        return str(a) == str(b)

    return a == b


def compare_dicts_keys(
    dict_1: dict,
    dict_2: dict,
    fields_to_compare: list = None,
    raise_exception: bool = False,
) -> dict:
    """
    Проверка одновременного существования полей из списка 'fields_to_compare'
    в словарях 'dict_1' и 'dict_2'.
    """

    errors = {}

    if fields_to_compare is None:
        fields_to_compare = set(list(dict_1.keys()) + list(dict_2.keys()))

    for field in fields_to_compare:
        dict_1_field_exists: bool = dict_field_exists(field=field, data=dict_1)
        dict_2_field_exists: bool = dict_field_exists(field=field, data=dict_2)

        if dict_1_field_exists != dict_2_field_exists:
            errors[field] = {
                "1": f"Поле '{field}' {'не ' if not dict_1_field_exists else ''} существует",
                "2": f"Поле '{field}' {'не ' if not dict_2_field_exists else ''} существует",
            }

    if errors and raise_exception:
        raise DataCompareException(message_dict=errors)

    return errors


def compare_dicts_values(
    dict_1: dict,
    dict_2: dict,
    fields_to_compare: list = None,
    raise_exception: bool = False,
) -> dict:
    """
    Сравнение значений полей из списка 'fields_to_compare' словарей 'dict_1' и 'dict_2'.
    """

    errors = {}

    if fields_to_compare is None:
        fields_to_compare = set(list(dict_1.keys()) + list(dict_2.keys()))

    for fields_pair in fields_to_compare:
        if isinstance(fields_pair, list) or isinstance(fields_pair, tuple):
            fields_pair = tuple(fields_pair)  # для хешируемости
            field_1 = fields_pair[0]
            field_2 = fields_pair[1]
        else:
            field_1 = fields_pair
            field_2 = fields_pair

        for field in [field_1, field_2]:
            if not isinstance(field, str):
                raise ValueError(f"Недопустимый тип поля: {type(field)}")

        _, value_1 = get_dict_field_value(field_1, dict_1)
        _, value_2 = get_dict_field_value(field_2, dict_2)

        equal = compare_values(value_1, value_2)
        if not equal:
            errors[fields_pair] = {
                "1": f"Значение поля '{field_1}': {value_1}",
                "2": f"Значение поля '{field_2}': {value_2}",
            }

    if errors and raise_exception:
        raise DataCompareException(message_dict=errors)

    return errors



from zs_utils.api.yandex_market.base_api import YandexMarketAPI


class UpdateYandexOfferPriceAPI(YandexMarketAPI):
    description = "Обновление цен предложений Yandex Market"
    http_method = "POST"
    resource_method = "campaigns/{shop_id}/offer-prices/updates"
    required_params = [
        "shop_id",
    ]



import magic
from io import BytesIO

from zs_utils.data.enums import CONTENT_TYPE_TO_FILE_EXTENSION


def get_file_extension(buffer: BytesIO = None, path: str = None) -> str:
    if path:
        content_type = magic.from_file(path, mime=True)
    elif buffer:
        content_type = magic.from_buffer(buffer.read(), mime=True)
        buffer.seek(0)
    else:
        raise ValueError("Необходимо указать один из параметров: 'buffer' или 'path'.")

    extension = CONTENT_TYPE_TO_FILE_EXTENSION.get(content_type)
    if not extension:
        raise ValueError(f"Не удалось конвертировать тип файла '{content_type}' в расширение.")
    return extension



from django_filters import filterset, filters
from zs_utils.api.models import APIRequestLog


__all__ = [
    "APIRequestLogFilter",
]


class APIRequestLogFilter(filterset.FilterSet):
    created = filters.DateTimeFromToRangeFilter()
    is_success = filters.BooleanFilter(method="get_is_success")
    response_time = filters.NumberFilter(lookup_expr="gte")
    url = filters.CharFilter(lookup_expr="icontains")

    class Meta:
        model = APIRequestLog
        fields = [
            "user",
        ]

    def get_is_success(self, queryset, name, value):
        if value:
            return queryset.filter(status_code__startswith="2")
        else:
            return queryset.exclude(status_code__startswith="2")



from django.contrib.auth import get_user_model
from rest_framework import serializers


__all__ = [
    "UserLightSerializer",
]

User = get_user_model()


class UserLightSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = [
            "id",
            "email",
            "is_staff",
            "first_name",
            "last_name",
            "full_name",
        ]



from model_utils import Choices


__all__ = [
    "FILE_EXTENSIONS",
    "FILE_EXTENSION_TO_CONTENT_TYPE",
    "CONTENT_TYPE_TO_FILE_EXTENSION",
]


FILE_EXTENSIONS = Choices(
    "csv",
    "pdf",
    "xlsx",
    "yml",
    "xml",
)

FILE_EXTENSION_TO_CONTENT_TYPE = {
    FILE_EXTENSIONS.csv: "text/csv",
    FILE_EXTENSIONS.pdf: "application/pdf",
    FILE_EXTENSIONS.xlsx: "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
    FILE_EXTENSIONS.yml: "application/xml",
    FILE_EXTENSIONS.xml: "application/xml",
}

CONTENT_TYPE_TO_FILE_EXTENSION = {
    **{value: key for key, value in FILE_EXTENSION_TO_CONTENT_TYPE.items()},
    "application/csv": FILE_EXTENSIONS.csv,
    "application/xml": FILE_EXTENSIONS.xml,
}



from zs_utils.api.ozon.base_api import OzonAPI


# --------------------------- FBO ---------------------------
# Отправления, которые обрабатывает Озон


class OzonGetFBOShipmentListAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/PostingAPI_GetFboPostingList
    """

    resource_method = "v2/posting/fbo/list"
    required_params = [
        "limit",
    ]
    allowed_params = ["offset", "dir", "filter"]


# --------------------------- FBS ---------------------------
# Отправления, которые обрабатывает пользователь


class OzonGetFBSShipmentListAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/PostingAPI_GetFbsPostingListV3
    """

    resource_method = "v3/posting/fbs/list"
    required_params = [
        "limit",
    ]
    allowed_params = ["offset", "dir", "filter"]


class OzonGetFBSShipmentAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/PostingAPI_GetFbsPostingV3
    """

    resource_method = "v3/posting/fbs/get"
    required_params = [
        "posting_number",
    ]
    allowed_params = ["with"]


class OzonCreateFBSShipmentAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/PostingAPI_ShipFbsPostingV3
    """

    resource_method = "v4/posting/fbs/ship"
    required_params = [
        "packages",
        "posting_number",
    ]
    allowed_params = ["with"]


class OzonGetLabelAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/PostingAPI_PostingFBSPackageLabel
    """

    resource_method = "v2/posting/fbs/package-label"
    required_params = ["posting_number"]


class OzonShipmentDeliveringAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/PostingAPI_FbsPostingDelivering
    """

    resource_method = "v2/fbs/posting/delivering"
    required_params = ["posting_number"]


class OzonShipmentDeliveredAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/PostingAPI_FbsPostingDelivered
    """

    resource_method = "v2/fbs/posting/delivered"
    required_params = ["posting_number"]


class OzonShipmentLastMileAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/PostingAPI_FbsPostingLastMile
    """

    resource_method = "v2/fbs/posting/last-mile"
    required_params = ["posting_number"]


class OzonSetShipmentTrackingNumberAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/PostingAPI_FbsPostingTrackingNumberSet
    """

    resource_method = "v2/fbs/posting/tracking-number/set"
    required_params = ["tracking_numbers"]


class OzonCancelShipmentAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/PostingAPI_CancelFbsPosting
    """

    resource_method = "v2/posting/fbs/cancel"
    required_params = [
        "posting_number",
        "cancel_reason_id",
    ]
    allowed_params = ["cancel_reason_message"]


class OzonGetShipmentCancelReasonsAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/PostingAPI_GetPostingFbsCancelReasonList
    """

    resource_method = "v2/posting/fbs/cancel-reason/list"



#!/usr/bin/env python
# coding=utf-8

from decimal import Decimal as D

from zs_utils.unit_converter.exceptions import UnConsistentUnitsError


class UnitPrefix(object):
    def __init__(self, symbol, name, factor):
        self.symbol = symbol
        self.name = name

        if isinstance(factor, str):
            self.factor = D(factor)
        elif isinstance(factor, D):
            self.factor = factor
        else:
            raise TypeError("factor need to be a 'string' or a" " 'decimal.Decimal' class")

    def __repr__(self):
        return "UnitPrefix(symbol='%s', name='%s', factor='%s')" % (self.symbol, self.name, self.factor)

    def is_same_factor(self, other_prefix):
        return self.factor == other_prefix.factor

    def __eq__(self, other_prefix):
        return (
            self.symbol == other_prefix.symbol and self.name == other_prefix.name and self.factor == other_prefix.factor
        )

    def __mul__(self, unit):
        if isinstance(unit, Unit):
            final_unit = Unit(
                symbol=self.symbol + unit.symbol,
                name=self.name + unit.name,
                L=unit.L,
                M=unit.M,
                T=unit.T,
                I=unit.I,
                THETA=unit.THETA,
                N=unit.N,
                J=unit.J,
                coef=self.factor * unit.coef,
                offset=unit.offset,
            )
            return final_unit
        else:
            raise TypeError("unsupported operand type(s) for : '%s' and '%s'" % (type(self), type(unit)))


class Unit(object):
    def __init__(
        self, symbol, name, plural_name=None, L=0, M=0, T=0, I=0, THETA=0, N=0, J=0, coef=D("1"), offset=D("0")
    ):
        self.symbol = symbol
        self.name = name
        self.plural_name = plural_name or name
        self.coef = coef
        self.offset = offset

        # Dimensional quantities
        # -----------------------
        self.L = L  # Length
        self.M = M  # Mass
        self.T = T  # Time
        self.I = I  # Electric current
        self.THETA = THETA  # Thermodynamic temperature
        self.N = N  # Amount of substance
        self.J = J  # Light intensity

    def __repr__(self):
        # TODO: Add a better representation including coef and offset.
        # TODO: Hide plotting 0 dimension
        l_units_r = ("m^%s", "kg^%s", "s^%s", "A^%s", "K^%s", "mol^%s", "cd^%s")
        units = (self.L, self.M, self.T, self.I, self.THETA, self.N, self.J)

        unit_r = [r % units[idx] for idx, r in enumerate(l_units_r) if units[idx]]
        return "*".join(unit_r)

    def is_same_dimension(self, other_unit):
        return (
            self.L == other_unit.L
            and self.M == other_unit.M
            and self.T == other_unit.T
            and self.I == other_unit.I
            and self.THETA == other_unit.THETA
            and self.N == other_unit.N
            and self.J == other_unit.J
        )

    def __eq__(self, other):
        return self.is_same_dimension(other) and self.coef == other.coef and self.offset == other.offset

    def __mul__(self, other):
        if isinstance(other, Unit):
            return self.__class__(
                symbol=self.symbol + "*" + other.symbol,
                name=self.name + "*" + other.name,
                L=self.L + other.L,
                M=self.M + other.M,
                T=self.T + other.T,
                I=self.I + other.I,
                THETA=self.THETA + other.THETA,
                N=self.N + other.N,
                J=self.J + other.J,
                coef=self.coef * other.coef,
                offset=self.offset + other.offset,
            )
        elif type(other) in (int, float, D):
            return Quantity(value=other, unit=self)
        else:
            raise TypeError("unsupported operand type(s) for : '%s' and '%s'" % (type(self), type(other)))

    def __pow__(self, power):
        if type(power) in (int, float, D):
            if self.offset:
                new_offset = self.offset ** D(power)
            else:
                new_offset = self.offset
            final_unit = self.__class__(
                symbol=self.symbol + "^" + str(power),  # TODO: attention manque des parenthèses etc..
                name=self.name + "^" + str(power),
                L=self.L * power,
                M=self.M * power,
                T=self.T * power,
                I=self.I * power,
                THETA=self.THETA * power,
                N=self.N * power,
                J=self.J * power,
                coef=self.coef ** D(power),
                offset=new_offset,
            )
            return final_unit
        else:
            raise TypeError("unsupported operand type(s) for : '%s' and '%s'" % (type(self), type(power)))

    def __truediv__(self, other):
        return self.__pow__(-1)

    def __rmul__(self, other):
        return self.__mul__(other)

    def __rtruediv__(self, other):
        return self.__truediv__(other)


class Quantity(object):
    def __init__(self, value, unit):
        if type(value) in (int, float, D):
            self.value = value
        else:
            raise TypeError("value must be an int, float or decimal class")

        if isinstance(unit, Unit):
            self.unit = unit
        else:
            raise TypeError("unit must be an Unit class")

    def convert(self, desired_unit: Unit):
        # Check dimension from current and desired units
        if not desired_unit.is_same_dimension(self.unit):
            raise UnConsistentUnitsError(desired_unit.name, self.unit.name)

        default_value = self.unit.offset + self.value * self.unit.coef
        desired_value = (-desired_unit.offset + default_value) / desired_unit.coef
        return self.__class__(value=desired_value, unit=self.unit)

    def __repr__(self):
        return str(self.value) + " " + str(self.unit)

    def __add__(self, other):
        if isinstance(other, Quantity):
            if self.unit == other.unit:
                return self.__class__(self.value + other.value, self.unit)

    def __sub__(self, other):
        if isinstance(other, Quantity):
            if self.unit == other.unit:
                return self.__class__(self.value - other.value, self.unit)

    def __mul__(self, other):
        if isinstance(other, Quantity):
            if self.unit == other.unit:
                return self.__class__(self.value * other.value, self.unit * other.unit)

    def __truediv__(self, other):
        if isinstance(other, Quantity):
            if self.unit == other.unit:
                return self.__class__(self.value / other.value, self.unit / other.unit)

    def __radd__(self, other):
        return self.__add__(other)

    def __rsub__(self, other):
        return self.__sub__(other)

    def __rmul__(self, other):
        return self.__mul__(other)

    def __rtruediv__(self, other):
        return self.__truediv__(other)



from django.conf import settings

from zs_utils.api.base_api import BaseAPI


class AmocrmAPI(BaseAPI):
    production_api_url = settings.AMOCRM_API_URL
    drop_empty_params = True

    def __init__(self, access_token: str, **kwargs):
        super().__init__(**kwargs)

        self.access_token = access_token

    @property
    def headers(self) -> dict:
        return {"Authorization": f"Bearer {self.access_token}"}



from zs_utils.api.ebay.base_api import EbayTradingAPI


__all__ = [
    "GetEbayCategoryFeatures",
    "GetCategorySpecifics",
    "GetTransportCategoryAspectsVS",
]


class GetEbayCategoryFeatures(EbayTradingAPI):
    """
    Docs:
    https://developer.ebay.com/devzone/xml/docs/reference/ebay/GetCategoryFeatures.html
    """

    method_name = "GetCategoryFeatures"

    def get_params(self, category_id=None, feature_ids=None, **kwargs):
        return {
            "AllFeaturesForCategory": False,
            "CategoryID": category_id,
            "FeatureID": feature_ids,
            "ViewAllNodes": True,
            "DetailLevel": "ReturnAll",
        }


class GetCategorySpecifics(EbayTradingAPI):
    """
    Docs:
    https://developer.ebay.com/devzone/xml/docs/reference/ebay/GetCategorySpecifics.html
    """

    method_name = "GetCategorySpecifics"

    def get_params(self, category_id, **kwargs):
        return {
            "CategoryID": category_id,
        }


class GetTransportCategoryAspectsVS(GetCategorySpecifics):
    def __init__(self, *args, **kwargs):
        kwargs["domain_code"] = "EBAY_MOTORS_US"
        return super().__init__(*args, **kwargs)



from zs_utils.api.ebay.base_api import EbayAPI


class CreateShippingFulfillment(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/fulfillment/resources/order/shipping_fulfillment/methods/createShippingFulfillment
    """

    http_method = "POST"
    resource_method = "sell/fulfillment/v1/order/{orderId}/shipping_fulfillment"
    required_params = ["orderId"]


class GetShippingFulfillment(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/fulfillment/resources/order/shipping_fulfillment/methods/getShippingFulfillment
    """

    http_method = "GET"
    resource_method = "sell/fulfillment/v1/order/{orderId}/shipping_fulfillment/{fulfillmentId}"
    required_params = ["orderId", "fulfillmentId"]


class GetShippingFulfillments(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/fulfillment/resources/order/shipping_fulfillment/methods/getShippingFulfillments
    """

    http_method = "GET"
    resource_method = "sell/fulfillment/v1/order/{orderId}/shipping_fulfillment"
    required_params = ["orderId"]



from zs_utils.api.wildberries.base_api import WildberriesAPI


class GetWildberriesNomenclatureList(WildberriesAPI):
    """
    https://openapi.wb.ru/content/api/ru/#tag/Prosmotr/paths/~1content~1v2~1get~1cards~1list/post
    """

    http_method = "POST"
    resource_method = "content/v2/get/cards/list"
    required_params = ["settings"]


class CreateWildberriesBarcodes(WildberriesAPI):
    """
    https://openapi.wb.ru/content/api/ru/#tag/Zagruzka/paths/~1content~1v2~1barcodes/post
    """

    http_method = "POST"
    resource_method = "content/v2/barcodes"
    required_params = ["count"]


class GetWildberriesFailedToUploadNomenclatureList(WildberriesAPI):
    """
    https://openapi.wb.ru/content/api/ru/#tag/Prosmotr/paths/~1content~1v2~1cards~1error~1list/get
    """

    http_method = "GET"
    resource_method = "content/v2/cards/error/list"


class CreateWildberriesNomenclature(WildberriesAPI):
    """
    https://openapi.wb.ru/content/api/ru/#tag/Zagruzka/paths/~1content~1v2~1cards~1upload/post
    """

    http_method = "POST"
    resource_method = "content/v2/cards/upload"
    required_params = ["subjectID", "variants"]


class UpdateWildberriesNomenclature(WildberriesAPI):
    """
    https://openapi.wb.ru/content/api/ru/#tag/Zagruzka/paths/~1content~1v2~1cards~1update/post
    """

    http_method = "POST"
    resource_method = "content/v2/cards/update"
    array_payload = True


class AddWildberriesNomenclaturesToCard(WildberriesAPI):
    """
    https://openapi.wb.ru/content/api/ru/#tag/Zagruzka/paths/~1content~1v2~1cards~1upload~1add/post
    """

    http_method = "POST"
    resource_method = "content/v2/cards/upload/add"
    required_params = [
        "imtID",
        "cardsToAdd",
    ]


class UpdateWildberriesNomenclatureImages(WildberriesAPI):
    """
    https://openapi.wb.ru/content/api/ru/#tag/Mediafajly/paths/~1content~1v3~1media~1save/post
    """

    http_method = "POST"
    resource_method = "content/v3/media/save"
    required_params = [
        "nmId",
        "data",
    ]


class GetWildberriesTrashListings(WildberriesAPI):
    """
    https://openapi.wb.ru/content/api/ru/#tag/Korzina/paths/~1content~1v2~1get~1cards~1trash/post
    """

    http_method = "POST"
    resource_method = "content/v2/get/cards/trash"
    allowed_params = ["settings"]


class DeleteWildberriesNomenclature(WildberriesAPI):
    """
    https://openapi.wb.ru/content/api/ru/#tag/Korzina/paths/~1content~1v2~1cards~1delete~1trash/post
    """

    http_method = "POST"
    resource_method = "content/v2/cards/delete/trash"
    allowed_params = ["nmIDs"]



__all__ = [
    "OutletsField",
]


class OutletsField:
    _outlets = []

    @property
    def outlets(self):
        return self._outlets

    @outlets.setter
    def outlets(self, outlets: list):
        self._outlets = outlets if outlets else []



# import argparse
# import os
# import time

# from pysftp import CnOpts, Connection

# cnopts = CnOpts()
# cnopts.hostkeys = None

# credentials = {
#     "host": "mip.ebay.com",
#     "port": 22,
#     "username": "zone-smart",
#     "password": "v^1.1#i^1#r^1#I^3#f^0#p^3#t^Ul4xMF82OjQ5REY2QzJDQjM2RTVERDE5MDIyRTAwNDU3NUQ3MzVBXzNfMSNFXjI2MA==",
# }

# mip_dirs = [
#     "availability",
#     "deleteInventory",
#     "distribution",
#     "inventory",
#     "location",
#     "order",
#     "orderfulfillment",
#     "product",
#     "reports",
# ]


# parser = argparse.ArgumentParser(description="Получение результатов загрузки eBay feed")


# class StoreNameValuePair(argparse.Action):
#     def __call__(self, parser, namespace, values, option_string=None):
#         n, v = values.split("=")
#         setattr(namespace, n, v)


# parser.add_argument("file_name", action=StoreNameValuePair, help="Имя файла")
# parser.add_argument(
#     "localpath",
#     action=StoreNameValuePair,
#     help="Путь на локальной машине для загруженного файла",
# )
# parser.add_argument(
#     "mip_dir",
#     action=StoreNameValuePair,
#     help="Папка на сервере mip.ebay.com, в которую был загружен feed-файл",
# )
# args = parser.parse_args()

# file_name = args.file_name
# print("Имя ранее загруженного файла:", file_name)

# localpath = os.path.join(os.path.dirname(__file__), args.localpath, file_name)
# localpath_dir = os.path.dirname(localpath)
# print("Результаты сохранятся в файл:", localpath)
# if not os.path.isdir(localpath_dir):
#     raise AttributeError(f"Указанная папка {localpath_dir} не существует на локальной машине")

# mip_dir = args.mip_dir
# print("Папка для поиска результатов в системе eBay:", mip_dir)
# if mip_dir not in mip_dirs:
#     raise AttributeError(
#         f"Недопустимая папка для поиска результатов загрузки: {mip_dir}\n" f"Допустимые папки: {mip_dirs}"
#     )


# def callback(ready, total):
#     print(f"File was {'un' if ready!=total else ''}successfully downloaded:")
#     print(f"{ready} of {total} bytes downloaded")


# def get_similar(filename, dirname):
#     print("Поиск файлов в", dirname)
#     print("Содержимое папки:", sftp.listdir(dirname))
#     files = [
#         os.path.join(dirname, file)
#         for file in sftp.listdir(dirname)
#         if (os.path.splitext(filename)[0] in file) and file.endswith(".csv")
#     ]
#     return files


# def get_last_modified(files):
#     dt_objs = []
#     for file in files:
#         dt_str = "-".join(os.path.basename(file).split("-")[1:7])
#         dt_obj = time.strptime(dt_str, "%b-%d-%Y-%H-%M-%S")
#         dt_objs.append((dt_obj, file))
#     if dt_objs:
#         return max(dt_objs, key=lambda x: x[0])[1]


# with Connection(**credentials, cnopts=cnopts) as sftp:
#     root_dir = os.path.join("store", mip_dir)
#     sftp.chdir(root_dir)

#     processed_files = []
#     for date_dir in sftp.listdir("output"):
#         processed_files += get_similar(file_name, os.path.join("output", date_dir))

#     unprocessed_files = []
#     if "inprogress" in sftp.listdir(""):
#         # возможно папка тоже содержит вложенные папки по датам
#         unprocessed_files = get_similar(file_name, "inprogress")

#     found_files = processed_files + unprocessed_files
#     print("Все найденные файлы:", found_files)

#     last_modified = get_last_modified(found_files)
#     print("Результаты последней загрузки:", last_modified)
#     if "output" in last_modified:
#         remotepath = last_modified
#     else:
#         print(f"Файл {last_modified} находится в обработке системой eBay, результаты пока недоступны.")
#         remotepath = get_last_modified(filter(lambda file_path: "output" in file_path, found_files))
#         if remotepath:
#             print(f"Загружен последний обработанный системой eBay файл с тем же именем {remotepath}.")
#         else:
#             print(f"Обработанных системой eBay файлов с тем же именем {file_name} не найдено.")

#     if remotepath:
#         sftp.get(remotepath=remotepath, localpath=localpath, callback=callback)
#         print(f"Файл {remotepath} загружен в {localpath}.")
#     else:
#         print("Файл не был загружен.")



from zs_utils.api.aliexpress.base_api import AliexpressAPI


class GetShippingTemplateListAPI(AliexpressAPI):
    resource_method = "aliexpress.freight.redefining.listfreighttemplate"


class GetShippingTemplateInfoAPI(AliexpressAPI):
    resource_method = "aliexpress.freight.redefining.getfreightsettingbytemplatequery"
    required_params = ["template_id"]



import datetime
from dateutil.parser import parse

from zs_utils.api.ebay.base_api import EbayAPI


class PaymentDisputeAPI(EbayAPI):
    production_api_url = "https://apiz.ebay.com/"
    sandbox_api_url = "https://apiz.sandbox.ebay.com/"


class GetPaymentDispute(PaymentDisputeAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/fulfillment/resources/payment_dispute/methods/getPaymentDispute
    """

    http_method = "GET"
    resource_method = "sell/fulfillment/v1/payment_dispute/{payment_dispute_id}"
    required_params = ["payment_dispute_id"]


class FetchEvidenceContent(PaymentDisputeAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/fulfillment/resources/payment_dispute/methods/fetchEvidenceContent
    """

    http_method = "GET"
    resource_method = "sell/fulfillment/v1/payment_dispute/{payment_dispute_id}/fetch_evidence_content"
    required_params = ["payment_dispute_id", "evidence_id", "file_id"]


class GetActivities(PaymentDisputeAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/fulfillment/resources/payment_dispute/methods/getActivities
    """

    http_method = "GET"
    resource_method = "sell/fulfillment/v1/payment_dispute/{payment_dispute_id}/activity"
    required_params = ["payment_dispute_id"]


class GetPaymentDisputeSummaries(PaymentDisputeAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/fulfillment/resources/payment_dispute/methods/getPaymentDisputeSummaries
    """

    http_method = "GET"
    resource_method = "sell/fulfillment/v1/payment_dispute/payment_dispute_summary"
    allowed_params = [
        "order_id",
        "buyer_username",
        "open_date_from",
        "open_date_to",
        "payment_dispute_status",
        "limit",
        "offset",
    ]
    MAX_LIMIT = 200

    def get_clean_params(self, params: dict) -> dict:
        clean_params = super().get_clean_params(params)

        if clean_params.get("open_date_to"):
            clean_params["open_date_to"] = self.clean_open_date_to(value=clean_params["open_date_to"])

        if clean_params.get("open_date_from"):
            clean_params["open_date_from"] = self.clean_open_date_from(value=clean_params["open_date_from"])

        return clean_params

    def clean_open_date_to(self, value):
        value = parse(value)

        now = datetime.datetime.now()
        if now < value:
            value = now
        elif (now - value).days >= 18 * 30:
            raise self.exception_class(
                'Разница между датой "open_date_to" и настоящим моментом не должна превышать 18 месяцев.'
            )

        return value

    def clean_open_date_from(self, value):
        value = parse(value)

        now = datetime.datetime.now()
        if now <= value:
            raise self.exception_class('Дата "open_date_from" должна быть более ранней, чем сегодняшняя дата.')
        elif (now - value).days >= 18 * 30:
            raise self.exception_class(
                'Разница между датой "open_date_from" и настоящим моментом не должна превышать 18 месяцев.'
            )

        return value
